
! DCFFTMR
! complex number fast fourier transform(FFT) with mixed radix
! author..... Hiroshi KOBAYASHI		JUNE, 1999
! corrected MAR, 2001

! subprograms are followings
! DCFFTMR -------- subroutine for interface
! rsvfft ------- recursive subroutine which is called by DCFFTMR
!				 to execute FFT using complex number

! subroutine DCFFTMR(cr, ci, n,isn,icon)
	! cr	[input and output], double precision
	!		1 dim array of real part of complex data cr(0:n-1)
	! ci	[input and output], double precision
	!		1 dim array of imaginary part of complex data ci(0:n-1)
	! n		[input], integer
	!		number of elements of cr(:), ci(:)
	! isn	[input], integer
	!		indicates transform(isn=+1) or inverse transform(isn=-1)
	! icon	[output], integer
	!		condition code( 0:no error, 1:n<=1, 2:isn/=+-1)

! recursive subroutine rsvfft(n, phase, c, tmp)
	! n		[input], integer
	!		number of elements of c(:) and tmp(:) to calculate
	! phase	[input], double precision
	!		phase of factor
	! c		[input], double complex
	!		1 dim complex array of data c(0:*)
	!		size depends upon real argument
	! tmp	[output], double complex
	!		1 dim temporary complex array tmp(0:*)
	!		size depends upon real argument


subroutine DCFFTMR(cr, ci, n,isn,icon)
	implicit none

	! Specify that DCFFTMR is exported to a DLL
	! and that the external name is 'DCFFTMR'
	!DEC$ ATTRIBUTES DLLEXPORT :: DCFFTMR
	!DEC$ ATTRIBUTES ALIAS:'DCFFTMR' :: DCFFTMR

!-----dummy arguments-----
	integer n, isn, icon
	double precision, dimension(0:(n-1)):: cr, ci

!-----local variables-----
	double complex, dimension(0:(n-1)):: cin, tmpin
	double precision phase
	integer i, nh

!-----calculation---------
	if(n<=1) then
		icon = 1
		return
	end if

	if((isn/=1).eqv.(isn/=-1)) then
		icon = 2
		return
	end if

	cin = dcmplx(cr,ci)
	phase = -isn*4.0d0*datan(1.0d0)*2.0d0/n

	call rsvfft(n, phase, cin, tmpin)

	cr = DBLE(cin)
	ci = DIMAG(cin)
	icon = 0
	return
end subroutine DCFFTMR


recursive subroutine rsvfft(n, phase, c, tmp)
	implicit none

!-----dummy arguments-----
	integer n
	double precision phase
	double complex, dimension(0:*):: c, tmp

!-----local variables-----
	integer rd, nrd, j, m, r
	double complex xc, w

!-----calculation---------
	if(n <= 1) return
	rd = 2
	do while(rd*rd <= n)
		if(mod(n,rd) == 0) exit
		rd = rd + 1
	end do

	if(mod(n,rd) /= 0) rd = n
	nrd = n/rd

	do j = 0, nrd-1
		do m = 0, rd-1
			xc = c(j)
			do r = nrd, n-1, nrd
				w = dcmplx(DCOS(phase*m*r),DSIN(phase*m*r))
				xc = xc + w*c(r+j)
			end do
			w = dcmplx(DCOS(phase*m*j),DSIN(phase*m*j))
			tmp(m*nrd+j) = xc * w
		end do
	end do

	do r = 0, n-1, nrd
		call rsvfft(nrd, phase*rd, tmp(r), c)
	end do

	do j = 0, nrd-1
		do m = 0, rd-1
			c(rd*j+m) = tmp(nrd*m+j)
		end do
	end do

end subroutine rsvfft
! End of file