
! DRFFTMR
! real number fast fourier transform(FFT) with mixed radix
! author..... Hiroshi KOBAYASHI		JUNE, 1999
! corrected MAR, 2001

! subprograms are followings
! DRFFTMR -------- subroutine for interface
! rsvfft ------- recursive subroutine which is called by DRFFTMR
!				 to execute FFT using complex number

! subroutine DRFFTMR(x,n,isn,icon)
	! x		[input and output], double precision
	!		1 dim array of data x(0:n-1)
	! n		[input], integer
	!		number of elements of x(:)
	! isn	[input], integer
	!		indicates transform(isn=+1) or inverse transform(isn=-1)
	! icon	[output], integer
	!		condition code( 0:no error, 1:n<=1, 2:isn/=+-1)

! recursive subroutine rsvfft(n, phase, c, tmp)
	! n		[input], integer
	!		number of elements of c(:) and tmp(:) to calculate
	! phase	[input], double precision
	!		phase of factor
	! c		[input], double complex
	!		1 dim complex array of data c(0:*)
	!		size depends upon real argument
	! tmp	[output], double complex
	!		1 dim temporary complex array tmp(0:*)
	!		size depends upon real argument


subroutine DRFFTMR(x,n,isn,icon)
	implicit none

	! Specify that DRFFTMR is exported to a DLL
	! and that the external name is 'DRFFTMR'
	!DEC$ ATTRIBUTES DLLEXPORT :: DRFFTMR
	!DEC$ ATTRIBUTES ALIAS:'DRFFTMR' :: DRFFTMR

!-----dummy arguments-----
	integer n, isn, icon
	double precision, dimension(0:(n-1)):: x

!-----local variables-----
	double complex, dimension(0:(n-1)):: cin, tmpin
	double precision phase
	integer i, nh

!-----calculation---------
	! if (n<=1) then stop calculation
	if(n<=1) then
		icon = 1
		return
	end if
	if(isn==1) then
		phase = -4.0d0*datan(1.0d0)*2.0d0/n
		cin = dcmplx(x,0.0d0)
	else if(isn==-1) then
		phase = 4.0d0*datan(1.0d0)*2.0d0/n
		if(mod(n,2)==0) then
			nh = n/2
			cin(0) = dcmplx(0.5d0*x(0),0.0d0)
			cin(nh) = dcmplx(0.5d0*x(nh),0.0d0)
			if(n>=4) then
				do i = 1, nh-1
					cin(i) = dcmplx(0.5d0*x(2*i),-0.5d0*x(2*i+1))
					cin(n-i) = dconjg(cin(i))
				end do
			end if
		else
			nh = (n-1)/2
			cin(0) = dcmplx(0.5d0*x(0),0.0d0)
			do i = 1, nh
				cin(i) = dcmplx(0.5d0*x(2*i-1), -0.5d0*x(2*i))
				cin(n-i) = dconjg(cin(i))
			end do
		end if
	else
		! if(isn/=+-1) stop calculation
		icon = 2
		return
	end if

	call rsvfft(n, phase, cin, tmpin)

	if(isn==1) then
		if(mod(n,2)==0) then
			nh = n/2
			x(0) = 2.0d0*DBLE(cin(0))
			x(1) = 2.0d0*DBLE(cin(nh))
			do i = 1 ,nh-1
				x(2*i) = 2.0d0*DBLE(cin(i))
				x(2*i+1) = -2.0d0*DIMAG(cin(i))
			end do
		else
			nh = (n-1)/2
			x(0) = 2.0d0*DBLE(cin(0))
			do i = 1, nh
				x(2*i-1) = 2.0d0*DBLE(cin(i))
				x(2*i) = -2.0d0*DIMAG(cin(i))
			end do
		end if
	else if(isn==-1) then
		x = 2.0d0*DBLE(cin)
	end if

	icon = 0
	return

end subroutine DRFFTMR


recursive subroutine rsvfft(n, phase, c, tmp)
	implicit none

!-----dummy arguments-----
	integer n
	double precision phase
	double complex, dimension(0:*):: c, tmp

!-----local variables-----
	integer rd, nrd, j, m, r
	double complex xc, w

!-----calculation---------
	if(n <= 1) return
	rd = 2
	do while(rd*rd <= n)
		if(mod(n,rd) == 0) exit
		rd = rd + 1
	end do

	if(mod(n,rd) /= 0) rd = n
	nrd = n/rd

	do j = 0, nrd-1
		do m = 0, rd-1
			xc = c(j)
			do r = nrd, n-1, nrd
				w = dcmplx(DCOS(phase*m*r),DSIN(phase*m*r))
				xc = xc + w*c(r+j)
			end do
			w = dcmplx(DCOS(phase*m*j),DSIN(phase*m*j))
			tmp(m*nrd+j) = xc * w
		end do
	end do

	do r = 0, n-1, nrd
		call rsvfft(nrd, phase*rd, tmp(r), c)
	end do

	do j = 0, nrd-1
		do m = 0, rd-1
			c(rd*j+m) = tmp(nrd*m+j)
		end do
	end do

end subroutine rsvfft
! End of file